'''
Created on Dec 17, 2014

@author: marten
'''
import os
from basesoft.web.contrib.notification.controllers import NotificationController
from basesoft.web.contrib.scheduler.controllers import SchedulerServerController

from django.utils.translation import ugettext as _

from basesoft.web.contrib.i18n.models import Country, EFTA_COUNTRIES
from basesoft.web.contrib.license.models import License
from basesoft.web.core.input.validation.plugins import Validation

from basesoft.web.core.util.controllers import AbstractController
from basesoft.web.core.exceptions import PublicSecurityException, PublicUserException, PublicSystemException
from sqlalchemy.sql.expression import and_
from django.core.exceptions import ValidationError

from django.conf import settings
from we.member.models import WeHqppLicense, WeLpp, WeCompany
from we.tbs.models import WeTbsLicense


class UniqueVATNumber(Validation):
    def check(self, params={"db_session": None}):

        if self._value in [None, '']:
            return

        if params['db_session'] is None:
            from basesoft.web.core.util.db import Session
            db_session = Session()
        else:
            db_session = params['db_session']

        _filter = WeCompany.vat_no == self._value

        if params.has_key('company_id') and params['company_id'] != None:
            _filter = and_(_filter, WeCompany.id != params['company_id'])

        company = db_session.query(WeLpp).join(WeCompany, WeLpp.company_id == WeCompany.id).filter(_filter).first()

        if params.has_key('license_id') and params['license_id'] != None:
            _filter = and_(_filter, License.id != params['license_id'])

        a = db_session.query(License).with_polymorphic([WeHqppLicense, WeTbsLicense]).\
            join(WeCompany, License.company_id == WeCompany.id).filter(_filter).first()

        if company is not None or a is not None:
            raise ValidationError(_('A company is already registered with the provided VAT number.'))


class AbstractWeController(AbstractController):

    def get_months_map(self):
        from basesoft.web.core import L10n_settings as l10n
        return l10n.default.MONTHS_LONG

    def get_months(self):
        months = self.get_months_map()
        rv = []

        for i in xrange(1, len(months) + 1):

            rv.append({
                'value': i,
                'label': months[i - 1]
            })

        return rv

    def get_tbs(self, tbs_id):
        return self.sa_session.query(WeTbsLicense).filter(WeTbsLicense.license_id==tbs_id).one()

    def get_tbs_licenses(self, tbs_id=None):
        """
        This should return the TBS licenses that the current user (member,tbs,operator,accountant) has access to.
        """
        if not self.has_permissions('we_tbs_license'):
            return self._get_own_tbs_licenses(tbs_id)

        q = self.sa_session.query(WeTbsLicense)
        if tbs_id is not None:
            q = q.filter(WeTbsLicense.id == tbs_id)

        rv = q.all()
        if not rv:
            raise PublicSecurityException(_("No such TBS."))

        return rv

    def _get_own_tbs_licenses(self, tbs_id=None):
        """
        This will return the tbs licenses where the current user is part of.
        """
        rv = []
        for group in self.get_active_groups():
            if not group.license.is_active:
                continue
            if isinstance(group.license, WeTbsLicense):
                if group.license not in rv and (tbs_id is None or tbs_id == group.license.id):
                    rv.append(group.license)

            elif isinstance(group.license, WeHqppLicense):
                lpps = self.get_local_producer_partners(group.license.id)
                for lpp in lpps:
                    if lpp.tbs not in rv and (tbs_id is None or tbs_id == lpp.tbs.id):
                        rv.append(lpp.tbs)
            # TODO: check for accountant, lpp etc.

        if not rv:
            raise PublicSecurityException(_("No access to TBS."))

        return rv

    def get_member_licenses(self, member_id=None):
        """
        If user doesn't have 'we_create_member_license' he is not an operator and fetch only own licenses
        else fetch all member licenses and if member_id is not None fetch that license
        :param member_id: int
        :return: List of licenses
        """

        if not self.has_permissions('we_create_member_license'):
            return self._get_own_member_licenses(member_id)

        q = self.sa_session.query(WeHqppLicense)
        if member_id is not None:
            q = q.filter(WeHqppLicense.id==member_id)
        return q.all()

    def _get_own_member_licenses(self, member_id=None):
        """
        This will return the member licenses where the current user is part of.
        """
        rv = []
        for group in self.get_active_groups():
            if not group.license.is_active:
                continue
            if not isinstance(group.license, WeHqppLicense):
                continue
            if group.license not in rv and (member_id is None or member_id == group.license.id):
                rv.append(group.license)

        if not rv:
            raise PublicSecurityException(_("No access to HQPP."))

        return rv

    def get_local_producer_partners(self, member_id=None, status=None, tbs_id=None):
        """
        This will return the local producer partners a logged in user has access to, for the supplied member_id and status.
        TODO: limit LPPs per account (#12)
        """

        members = self.get_member_licenses(member_id)
        rv = []
        for member in members:
            for lpp in member.lpps:
                if tbs_id is not None and lpp.tbs.id != tbs_id:
                    continue

                if (status is None and lpp.status == WeLpp.STATUS_ACTIVE) or status == lpp.status:
                    rv.append(lpp)
        return rv

    def get_local_producer_partner(self, lpp_id):
        lpp = self.sa_session.query(WeLpp).filter(WeLpp.id == lpp_id).first()
        if lpp is None:
            return None
        members = self.get_member_licenses()
        if lpp.hqpp not in members:
            raise PublicSecurityException(_('No access to local producer partner'))
        return lpp

    def get_lpp_with_all_tbses(self, company_id):
        """
        An LPP can be registered in multiple TBSes so we should use Company id for correct distinction
        and return all active records for that Company
        :param company_id: int
        :return: List
        """
        lpp = self.sa_session.query(WeLpp).filter(and_(WeLpp.company_id == int(company_id), WeLpp.status == WeLpp.STATUS_ACTIVE)).all()
        if len(lpp) == 0:
            return []

        records = []
        members = self.get_member_licenses()

        for lp in lpp:
            if lp.hqpp in members:
                records.append(lp)

        if len(records) == 0:
            raise PublicUserException(_('No access to local producer partner.'))
        return records

    def assert_local_producer_partner(self, lpp_id):
        lpp = self.get_local_producer_partner(lpp_id)
        if lpp is None:
            raise PublicSecurityException(_('No access to local producer partner'))

    def get_file_extension(self, file_name, file_hash):

        file_path = os.path.join(settings.TMP_FILE_REPOSITORY, file_hash)
        assert os.path.exists(file_path), 'file not found: %s' % file_path

        return os.path.splitext(file_name)

    def start_task(self, task, params):
        ssc = SchedulerServerController(self.sa_session)
        ssc.system_start_task(task, params)


class WeCoreController(AbstractWeController):

    def get_efta_countries(self):
        return self.sa_session.query(Country).filter(Country.iso_3166_a2.in_(EFTA_COUNTRIES)).all()


class WeNotificationController(NotificationController):
    pass
