# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

from sqlalchemy import Integer, Column, ForeignKey

from basesoft.web.contrib.license import models as license_models
from basesoft.sqlalchemy.table import IQTable
from sqlalchemy.orm import mapper

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables

def do_tables(metadata):
    """
    This function is called by Django middleware when needed.
    """
    global __tables_defined
    if __tables_defined:
        return

    tblWeOperatorLicense = IQTable('tblWeOperatorLicense', metadata,
        Column('license_id', Integer, ForeignKey('tblLicense.id', ondelete='CASCADE'), primary_key=True)
    )

    tables['tblWeOperatorLicense'] = tblWeOperatorLicense

    __tables_defined = True

def do_mappings():
    """
    This function is called by Django middleware when needed.
    """
    global __mapping_defined
    if __mapping_defined:
        return

    from we.core import models as core_models

    mapper(core_models.WeOperatorLicense, tables['tblWeOperatorLicense'],
        polymorphic_identity='WeOperatorLicense',
        inherits=license_models.License
    )

    __mapping_defined = True

def fill_data(conn):
    pass

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'WEEE Europe core')

def app_permissions():
    from django.utils.translation import ugettext as _
    return [

        license_models.Permission(u'we_operator_license', u"""
            This gives the right to create WE Operator licenses
            """, _("WE Operator license management")),

        license_models.Permission(u'we_manage_roles', u"""
            This gives the right to manage WE roles
            """, _("WE role management")),

    ]

def app_roles():
    return [
        {
            'name': 'WE Operator',
            'required_permission': 'manage_iqportal_roles',
            'permissions': [
                'view_dashboard','save_dashboard', 'distribute_group_permissions', 'distribute_license_permissions',
                'license_masterdata_change_approval','manage_license_configs', 'update_license_masterdata',
                'create_file', 'view_file', 'view_license', 'update_license', 'view_domain', 'manage_company_notes',
                'create_mail', 'update_mail', 'view_mail', # for account activation and lost password mail.
                'create_account_mail', 'update_account_mail', 'view_account_mail', 'delete_account_mail',
                'manage_announcements', 'receive_announcements', 'set_custom_query_preset',
                'create_account', 'update_account', 'account_activation_reset', 'manage_user_passwords',
                'manage_roles', 'we_manage_roles', 'apply_roles',
                'view_notification', 'archive_notification',
                'we_tbs_license',
                'we_add_company',
                'we_view_weee_category',
                'we_manage_cn8',
                'we_manage_tbs_product_level', 'we_view_tbs_product_level',
                'we_create_member_license', 'we_manage_member_license',
                'we_manage_lpp_companies',
                'we_manage_fee',
                'we_manage_declaration', 'we_view_declaration',
                'we_create_products', 'we_view_products',
                'switch_account',
                'we_manage_cn8_tbs_product_level'
            ],
            'license_configs': [],
            'account_configs': [
                ('basesoft.account.use.iconbar', 'False'),
            ]
        }
    ]

