# Copyright (c) 2016 BaseSoft B.V.
# See LICENSE for details.

import decimal

from pyws.functions.register import register

from basesoft.web.contrib.soap import soap_assert_logged_in
from basesoft.web.core.exceptions import PublicUserException
from basesoft.logging import getLogger

from we.declaration.controllers import WeDeclarationController

logging = getLogger('basesoft.web.contrib.order')

# Declaration CN8 Data
DeclarationCn8DataLineDict = {
    0: 'DeclarationCn8DataLineDict',
    'quantity': int,
    'weight': decimal.Decimal,
    'cn8': str,
}

DeclarationCn8DataDict = {
    0: 'DeclarationCn8DataDict',
    'producer_relation_id': int,
    'lines': [DeclarationCn8DataLineDict],
    'year': int,
    'month': int,
    'quarter': int,
    'bi-annual': int,
}

def register_soap_functions(server):

    @register(args=((int, None, ), (int, None, ), (int, None, ), (int, None, ), (int, None, )), return_type=[DeclarationCn8DataDict], to=server, needs_context=True)
    def get_declarations_by_cn8(tbs_id, year, month, quarter, bi_annual, context):
        logging.info('soap::get_declarations_by_cn8(%s, %s, %s, %s, %s)' % (tbs_id, year, month, quarter, bi_annual))

        if tbs_id is None or year is None:
            raise PublicUserException('Missing required field.')

        soap_assert_logged_in(context=context)

        c = WeDeclarationController(context.Session(), account_id=context.session['user_id'], request=context)
        return c.get_declarations_by_cn8(tbs_id, year, month, quarter, bi_annual)
