'''
Created on Jan 29, 2015

@author: marten
'''
from django.utils.translation import ugettext, ugettext_lazy as _
from basesoft.web.core.config import DefaultConfig
from basesoft.web.contrib.ticketing.config import TicketingLicenseConfig
from basesoft.web.core.input.validation import plugins
from we.tbs.models import WeTbsLicense

class WeTbsLicenseConfigs(DefaultConfig):
    permissions = ('we_tbs_license', )
    label = _('TBS settings')
    is_expanded = True

    def configs(self):
        return [{
            'name': 'we.tbs.declaration.period',
            'label': ugettext('Maximum period in which a declaration must be performed.'),
            'options': {
                'options': {
                    WeTbsLicense.DECLARATION_PERIOD_MONTHLY: ugettext('Monthly'),
                    WeTbsLicense.DECLARATION_PERIOD_QUARTERLY: ugettext('Quarterly'),
                    WeTbsLicense.DECLARATION_PERIOD_BIANNUALLY: ugettext('Bi-annually'),
                    WeTbsLicense.DECLARATION_PERIOD_YEARLY: ugettext('Yearly'),
                }
            }
        }, {
            'name': 'we.tbs.use.sales.price',
            'label': ugettext('Swiss TBSes use sales price to allocate SKU into a local category.'),
        }, {
            'name': 'we.tbs.require.end.user.distinction',
            'label': ugettext('Member needs to provide a TBS category distinction between b2b and b2c.'),
        }, {
            'name': 'we.tbs.use.negative.fee',
            'label': ugettext('Enable negative fee.'),
        }, {
            'name': 'we.tbs.declaration.closing.day',
            'label': ugettext('Declaration closing day')
        }]

    def get_form_behaviour(self, validators, values):

        closing_day_integer_params = {'unsigned': True, 'allow_zero': False}
        if not values.has_key('we.tbs.declaration.period') or values['we.tbs.declaration.period'] == None:
            validators['we.tbs.declaration.closing.day'] = [(plugins.Integer, closing_day_integer_params, )]
            return validators

        closing_day_integer_params['max_value'] = 27 # february
        validators['we.tbs.declaration.closing.day'] = [(plugins.Integer, closing_day_integer_params, )]

        return validators


license_configs = {
    'WeTbsLicense' : (WeTbsLicenseConfigs(), ),
}
